@@ -4,7 +4,7 @@
 #include <rtx_config.h>
 #include <iostream>
 #include <lttoolbox/alphabet.h>
-#include <lttoolbox/ltstr.h>
+#include <lttoolbox/ustring.h>
 #include <lttoolbox/transducer.h>
 
 #include <string>
@@ -15,8 +15,8 @@ using namespace std;
 
 struct PatternElement
 {
-  wstring lemma;
-  vector<wstring> tags;
+  UString lemma;
+  vector<UString> tags;
 };
 
 class PatternBuilder
@@ -31,27 +31,27 @@ private:
    * Attribute categories
    * name => regex
    */
-  map<wstring, wstring, Ltstr> attr_items;
+  map<UString, UString> attr_items;
 
   /**
    * Lists
    * name => { values }
    */
-  map<wstring, set<wstring, Ltstr>, Ltstr> lists;
+  map<UString, set<UString>> lists;
 
   /**
    * Global string variables
    * name => initial value
    */
-  map<wstring, wstring, Ltstr> variables;
+  map<UString, UString> variables;
 
   /**
    * Symbols marking ends of rules in pattern transducer
    */
   set<int> final_symbols;
 
-  map<int, vector<wstring>> lookahead;
-  map<wstring, set<wstring>> firstSet;
+  map<int, vector<UString>> lookahead;
+  map<UString, set<UString>> firstSet;
 
   /**
    * Alphabet of pattern transducer
@@ -67,9 +67,9 @@ private:
    * Lexicalized weights for rules
    * rule id => [ ( weight, processed pattern ) ... ]
    */
-  map<wstring, vector<pair<double, vector<vector<PatternElement*>>>>> lexicalizations;
+  map<UString, vector<pair<double, vector<vector<PatternElement*>>>>> lexicalizations;
 
-  map<int, pair<vector<wstring>, vector<vector<PatternElement*>>>> rules;
+  map<int, pair<vector<UString>, vector<vector<PatternElement*>>>> rules;
 
   //////////
   // TRANSDUCER PATH BUILDING
@@ -79,23 +79,23 @@ private:
    * Starting from base, add path for lemma
    * @return end state
    */
-  int insertLemma(int const base, wstring const &lemma);
+  int insertLemma(int const base, UString const &lemma);
 
   /**
    * Starting from base, insert each tag in tags
    * @return end state
    */
-  int insertTags(int const base, const vector<wstring>& tags);
+  int insertTags(int const base, const vector<UString>& tags);
 
   /**
-   * Generate symbol of the form L"<RULE_NUMBER:count>" to mark rule end
+   * Generate symbol of the form "<RULE_NUMBER:count>" to mark rule end
    */
   int countToFinalSymbol(const int count);
 
   /**
    * Build complete path
    */
-  void addPattern(vector<vector<PatternElement*>> pat, int rule, double weight, bool isLex);
+  void addPattern(const vector<vector<PatternElement*>>& pat, int rule, double weight, bool isLex);
 
   void buildLookahead();
 
@@ -103,31 +103,6 @@ private:
 
   void buildFallback();
 
-  //////////
-  // ATTRIBUTE COMPRESSION
-  //////////
-
-  struct TrieNode
-  {
-    wchar_t self;
-    vector<TrieNode*> next;
-  };
-
-  /**
-   * Construct tries for a set of inputs, return one for each initial character
-   */
-  vector<TrieNode*> buildTrie(vector<wstring> parts);
-
-  /**
-   * Convert trie to regex
-   */
-  wstring unbuildTrie(TrieNode* t);
-
-  /**
-   * Wrapper around buildTrie() and unbuildTrie()
-   */
-  wstring trie(vector<wstring> parts);
-
 public:
 
   //////////
@@ -136,41 +111,40 @@ public:
 
   // false: * = 1 or more tags, true: * = 0 or more tags
   /**
-   * If false, L"*" must match at least one tag, otherwise it can match 0
-   * Default: false
+   * If false, "*" must match at least one tag, otherwise it can match 0
    */
-  bool starCanBeEmpty;
+  bool starCanBeEmpty = false;
 
   /**
    * Number of global Chunk* variables to allocate space for
    */
-  unsigned int chunkVarCount;
+  unsigned int chunkVarCount = 0;
 
   /**
    * Debug names for input-time rules
    */
-  vector<wstring> inRuleNames;
+  vector<UString> inRuleNames;
 
   /**
    * Debug names for output-time rules
    */
-  vector<wstring> outRuleNames;
+  vector<UString> outRuleNames;
 
   PatternBuilder();
 
-  void addRule(int rule, double weight, vector<vector<PatternElement*>> pattern, vector<wstring> firstChunk, wstring name);
-  void addList(wstring name, set<wstring, Ltstr> vals);
-  void addAttr(wstring name, set<wstring, Ltstr> vals);
-  bool isAttrDefined(wstring name);
-  void addVar(wstring name, wstring val);
+  void addRule(int rule, double weight, const vector<vector<PatternElement*>>& pattern, const vector<UString>& firstChunk, const UString& name);
+  void addList(const UString& name, const set<UString>& vals);
+  void addAttr(const UString& name, const set<UString>& vals);
+  bool isAttrDefined(const UString& name);
+  void addVar(const UString& name, const UString& val);
   void loadLexFile(const string& fname);
-  void write(FILE* output, int longest, vector<pair<int, wstring>> inputBytecode, vector<wstring> outputBytecode);
+  void write(FILE* output, int longest, vector<pair<int, UString>> inputBytecode, vector<UString> outputBytecode);
 
   //////////
   // BYTECODE CONSTRUCTION
   //////////
-  wstring BCstring(const wstring& s);
-  wstring BCifthenelse(const wstring& cond, const wstring& yes, const wstring& no);
+  UString BCstring(const UString& s);
+  UString BCifthenelse(const UString& cond, const UString& yes, const UString& no);
 };
 
 #endif
