@@ -721,6 +721,9 @@ RTXCompiler::parsePatternElement(Rule* rule)
   vector<UString> pat;
   if(isNextToken('%'))
   {
+    if(rule->grab_all != -1) {
+      die("Cannot set the analysis clip source multiple times. (You have multiple % in your rule pattern.)"_u);
+    }
     rule->grab_all = rule->pattern.size()+1;
   }
   UString t1 = nextToken();
@@ -733,6 +736,10 @@ RTXCompiler::parsePatternElement(Rule* rule)
     t1 = "$"_u + parseIdent();
     if(!isNextToken(']')) die("expected closing bracket after lemma category"_u);
   }
+  else if(t1 == "|"_u)
+  {
+    die("Rule is missing output (use quotes if you were trying to use | as a lemma)"_u);
+  }
   if(isNextToken('@'))
   {
     pat.push_back(t1);
@@ -1753,8 +1760,7 @@ RTXCompiler::compileClip(Clip* c, UString _dest = ""_u)
       cur += compileTag(rule[i].second);
       if(i != 1)
       {
-        cur += JUMP;
-        cur += (UChar)check.size();
+        cur += compileJump(check);
       }
       check = cur + check;
     }
@@ -1904,6 +1910,22 @@ RTXCompiler::processMacroChoice(OutputChoice* mac, OutputChunk* arg)
   return ret;
 }
 
+UString
+RTXCompiler::compileJump(const UString& over)
+{
+  uint32_t n = over.size();
+  UString ret;
+  if (n < (2 << 16)) {
+    ret += JUMP;
+    ret += (UChar)n;
+  } else {
+    ret += LONGJUMP;
+    ret += (UChar)(n >> 16);
+    ret += (UChar)(n & 0xFFFF);
+  }
+  return ret;
+}
+
 UString
 RTXCompiler::processOutputChunk(OutputChunk* r)
 {
@@ -2362,8 +2384,7 @@ RTXCompiler::processOutputChoice(OutputChoice* choice)
     {
       act = processOutputChunk(choice->chunks[n-i]);
     }
-    act += JUMP;
-    act += (UChar)ret.size();
+    act += compileJump(ret);
     UString cond = processCond(choice->conds[n-i]);
     cond += JUMPONFALSE;
     cond += (UChar)act.size();
