@@ -609,6 +609,11 @@ private:
   OutputChunk* processMacroChunk(OutputChunk* mac, OutputChunk* arg);
   OutputChoice* processMacroChoice(OutputChoice* mac, OutputChunk* arg);
 
+  /**
+   * Compile either JUMP or LONGJUMP of over.length()
+   */
+  UString compileJump(const UString& over);
+
   /**
    * Compile a non-chunk output element
    * @param ch - the element
